var ansiStyles = require('../ansi-styles')
var supportsColor = require('./supports-color')

var escapeStringRegexp = str => str.replace(/[|\\{}()[\]^$+*?.]/g, '\\$&')

var defineProps = Object.defineProperties
var isSimpleWindowsTerm =
  process.platform === 'win32' && !(/^xterm/i).test(process.env.TERM)

function Chalk(options) {
  // detect mode if not set manually
  this.enabled = !options || options.enabled === undefined ?
    supportsColor :
    options.enabled
}

// use bright blue on Windows as the normal blue color is illegible
if (isSimpleWindowsTerm) {
  ansiStyles.blue.open = '\u001b[94m'
}

var styles = {}

Object.keys(ansiStyles).forEach(key => {
  ansiStyles[key].closeRe = new RegExp(
    escapeStringRegexp(ansiStyles[key].close),
    'g'
  )

  styles[key] = {
    get() {
      return build.call(this, this._styles ? this._styles.concat(key) : [key])
    },
  }
})

// eslint-disable-next-line func-names
var proto = defineProps(() => {}, styles)

function build(_styles) {
  var builder = function() {
    return applyStyle.apply(builder, arguments)
  }

  var self = this

  builder._styles = _styles

  Object.defineProperty(builder, 'enabled', {
    enumerable: true,
    get() {
      return self.enabled
    },
    set(v) {
      self.enabled = v
    },
  })

  // __proto__ is used because we must return a function, but there is
  // no way to create a function with a different prototype.
  /* eslint-disable no-proto */
  builder.__proto__ = proto

  return builder
}

function applyStyle() {
  // support varags, but simply cast to string in case there's only one arg
  var args = arguments
  var argsLen = args.length
  var str = argsLen !== 0 && String(arguments[0])

  if (argsLen > 1) {
    // don't slice `arguments`, it prevents v8 optimizations
    for (var a = 1; a < argsLen; a++) {
      str += ' ' + args[a]
    }
  }

  if (!this.enabled || !str) {
    return str
  }

  var nestedStyles = this._styles
  var i = nestedStyles.length

  // Turns out that on Windows dimmed gray text becomes invisible in cmd.exe,
  // see https://github.com/chalk/chalk/issues/58
  // If we're on Windows and we're dealing with a gray color, temporarily make 'dim' a noop.
  var originalDim = ansiStyles.dim.open
  if (
    isSimpleWindowsTerm &&
    (nestedStyles.indexOf('gray') !== -1 || nestedStyles.indexOf('grey') !== -1)
  ) {
    ansiStyles.dim.open = ''
  }

  while (i--) {
    var code = ansiStyles[nestedStyles[i]]

    // Replace any instances already present with a re-opening code
    // otherwise only the part of the string until said closing code
    // will be colored, and the rest will simply be 'plain'.
    str = code.open + str.replace(code.closeRe, code.open) + code.close

    // Close the styling before a linebreak and reopen
    // after next line to fix a bleed issue on macOS
    // https://github.com/chalk/chalk/pull/92
    str = str.replace(/\r?\n/g, code.close + '$&' + code.open)
  }

  // Reset the original 'dim' if we changed it to work around the Windows dimmed gray issue.
  ansiStyles.dim.open = originalDim

  return str
}

defineProps(Chalk.prototype, styles)

module.exports = new Chalk()
module.exports.styles = ansiStyles
module.exports.supportsColor = supportsColor
